/*
 * Copyright ? Matthew Chappee <matthew@orasoft.org>
 *             Paul Flinders <paul@orasoft.org>
 *             Brent Gilmore <brent@orasoft.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "sqlworksig.h"
#include "sqlworksrc.h"

void
on_ok_button_clicked(
    GtkButton       *button,
    gpointer         user_data)
{
    gtk_widget_destroy(GTK_WIDGET(user_data));
}


GtkWidget*
create_error_dialog (char *msg)
{
  GtkWidget *error_dialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *fixed1;
  GtkWidget *label1;
  GtkWidget *pixmap1;
  GtkWidget *dialog_action_area1;
  GtkWidget *ok_button;
  char      *err_icon;

  error_dialog = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (error_dialog), "error_dialog", error_dialog);
  gtk_window_set_title (GTK_WINDOW (error_dialog), "SQLWork Error");
  gtk_window_set_modal (GTK_WINDOW (error_dialog), TRUE);
  gtk_window_set_policy (GTK_WINDOW (error_dialog), FALSE, FALSE, FALSE);

  dialog_vbox1 = GTK_DIALOG (error_dialog)->vbox;
  gtk_object_set_data (GTK_OBJECT (error_dialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  fixed1 = gtk_fixed_new ();
  gtk_widget_show (fixed1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), fixed1, TRUE, TRUE, 0);

  label1 = gtk_label_new (msg);
  gtk_widget_show (label1);
  gtk_fixed_put (GTK_FIXED (fixed1), label1, 80, 0);
  gtk_widget_set_uposition (label1, 80, 0);
  gtk_widget_set_usize (label1, 240, 152);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  err_icon = g_malloc(strlen(sql_ginfo.iconpath) + 16);
  sprintf(err_icon,"%s/error.xpm", sql_ginfo.iconpath);

  pixmap1 = sql_create_pixmap (error_dialog, err_icon);
  gtk_widget_show (pixmap1);
  gtk_fixed_put (GTK_FIXED (fixed1), pixmap1, 16, 64);
  gtk_widget_set_uposition (pixmap1, 16, 64);
  gtk_widget_set_usize (pixmap1, 21, 21);
  g_free(err_icon);

  dialog_action_area1 = GTK_DIALOG (error_dialog)->action_area;
  gtk_object_set_data (GTK_OBJECT (error_dialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_set_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  ok_button = gtk_button_new_with_label ("OK");
  gtk_widget_show (ok_button);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), ok_button, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
                      GTK_SIGNAL_FUNC (on_ok_button_clicked),
                      error_dialog);

  return error_dialog;
}

void
file_open_error(char *fname, int e, char *rw)
{
    GtkWidget   *error_dialog;
    char        *msg;

    msg = g_malloc(strlen(fname) + strlen(strerror(e)) + 32);
    
    sprintf(msg, "Can't open %s for %s\n%s", fname, rw, strerror(e));
    error_dialog = create_error_dialog(msg);
    gtk_widget_show(error_dialog);
    g_free(msg);
}
