/*
 * Copyright ? Matthew Chappee <matthew@orasoft.org>
 *             Paul Flinders <paul@orasoft.org>
 *             Brent Gilmore <brent@orasoft.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "sqlworksrc.h"
#include "sqlworksig.h"
#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sqlcpr.h>
#include <time.h>


#define MAX_ITEMS         300
#define MAX_VNAME_LEN     30
#define MAX_INAME_LEN     30
#ifndef NULL
#define NULL  0
#endif

EXEC SQL INCLUDE sqlda;
EXEC SQL INCLUDE sqlca;
SQLDA *bind_dp;
SQLDA *select_dp;

EXEC SQL BEGIN DECLARE SECTION;
   char    dyn_statement[1024];
   EXEC SQL VAR dyn_statement IS STRING(1024);
   const char *sql_cstring;
EXEC SQL END DECLARE SECTION;


GtkWidget*
sql_get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
sql_set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
sql_add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
sql_check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
sql_create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = sql_check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = sql_check_file_exists ("", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

void sql_sql_error()
{
char errmsg[200],omsg[200],*buf[2];
  sprintf(omsg,"\r\n\r\n%.70s\r\n",sqlca.sqlerrm.sqlerrmc);
  printf("\n%s\n",omsg);
  gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), NULL, NULL, NULL,omsg, strlen(omsg));
  sql_rebuildclist();
  buf[0]=omsg;
  buf[1]=NULL;
  gtk_clist_append( (GtkCList *) sql_mainwin.sql_clist2, buf);
  sql_ginfo.doingquery = 0;
} 

sql_alloc_descriptors(size, max_vname_len, max_iname_len)
int size;
int max_vname_len;
int max_iname_len;
{
    int i;

    if ((bind_dp =
            sqlald(size, max_vname_len, max_iname_len)) == (SQLDA *) 0)
    {
        fprintf(stderr,
            "Cannot allocate memory for bind descriptor.");
        return -1;  
    } 

    if ((select_dp =
        sqlald (size, max_vname_len, max_iname_len)) == (SQLDA *) 0)
    {
        fprintf(stderr,
            "Cannot allocate memory for select descriptor.");
        return -1;
    }
    select_dp->N = MAX_ITEMS;

    /* Allocate the pointers to the indicator variables, and the
       actual data. */
    for (i = 0; i < MAX_ITEMS; i++) {
        bind_dp->I[i] = (short *) malloc(sizeof (short)); 
        select_dp->I[i] = (short *) malloc(sizeof(short));
        bind_dp->V[i] = (char *) malloc(1); 
        select_dp->V[i] = (char *) malloc(1);
    }
       
    return 0;
}


int sql_get_dyn_statement()
{
    char *cp;
    char linebuf[256],*buf2;
    int startpos=0,endpos=0;

  cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text5)));
     if(cp==NULL){
       printf("\r\nOut of Memory\r\n");
       exit(0);
     }

  if(GTK_EDITABLE (sql_mainwin.sql_text5)->has_selection){
    startpos=GTK_EDITABLE (sql_mainwin.sql_text5)->selection_start_pos;
    endpos=GTK_EDITABLE (sql_mainwin.sql_text5)->selection_end_pos;

	/* Fix provided by Paul Flinders <P.Flinders@ftel.co.uk> */
	/* If user selected backwards, reverse the start and end pos */
    if (startpos > endpos) {
 		startpos=GTK_EDITABLE (sql_mainwin.sql_text5)->selection_end_pos;
 		endpos=GTK_EDITABLE (sql_mainwin.sql_text5)->selection_start_pos;
    }
    cp=gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text5),(gint) startpos,(gint) endpos);
  }else{

    cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text5),(gint) 0,
      (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text5)));
      
  }    
	buf2=(char *)malloc(strlen(cp)+1);
        if(buf2==NULL){
          printf("\r\nOut of Memory\r\n");
          exit(0);
        }
	
	strcpy(buf2,cp);
	free(cp);
	
        strcat(dyn_statement, nosemi(nocrlfe(nocrlfb(sse(ssb(buf2))))));	
	free(buf2);
	
	if(sql_ginfo.totalhistory == 19){
	  sql_ginfo.totalhistory = 0;
	  sql_ginfo.freemem = 1;  
	}
	  
	if(sql_ginfo.freemem)
	  free(sql_ginfo.sqlhistory[sql_ginfo.totalhistory]);
	  
	sql_ginfo.sqlhistory[sql_ginfo.totalhistory]=(char *)malloc(strlen(dyn_statement)+1);
          if(sql_ginfo.sqlhistory[sql_ginfo.totalhistory]==NULL){
            printf("\r\nOut of Memory\r\n");
            exit(0);
          }
	      
	strcpy(sql_ginfo.sqlhistory[sql_ginfo.totalhistory],dyn_statement);
	sql_ginfo.totalhistory++;

        return 0;
}


/* This function is obsolete */
void sql_set_bind_variables()
{
    int i, n;
    char bind_var[64];

/*
    EXEC SQL WHENEVER SQLERROR DO sql_sql_error();

    bind_dp->N = MAX_ITEMS;  
    EXEC SQL DESCRIBE BIND VARIABLES FOR S INTO bind_dp;

    if (bind_dp->F < 0)
    {
        printf ("\nToo many bind variables (%d), maximum is %d\n.",
                    -bind_dp->F, MAX_ITEMS);
        return;
    }

    bind_dp->N = bind_dp->F;  
*/
     
  return;
}


int sql_doconnect()
{
EXEC SQL BEGIN DECLARE SECTION;
char *buf,buf2[500],*buf3,*cstring;
EXEC SQL END DECLARE SECTION;
GdkFont *fixed_font;

  EXEC SQL WHENEVER SQLERROR GOTO breakout;

  strcat(dyn_statement,"  ");
  buf=strtok(dyn_statement," ");
  buf=strtok(NULL," ");
  cstring=buf;

  EXEC SQL CONNECT :cstring;

  strcat(buf,"/");
  buf3=strtok(buf,"/");
  sprintf(buf2,"Connected as %s",buf3);

  strcpy(sql_ginfo.globallogin,cstring);

  fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
  gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), fixed_font, NULL, NULL,buf2, strlen(buf2));
    
  gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar1),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar1), ""), " ");
  gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar2),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar2), ""), " ");  
    
 return 0;
	              
breakout:
 sql_sql_error(); 
 return 1;
}

/*int doforkexec()
{
pid_t pid;
char *buf[2];

  pid = fork();

  if (pid == 0)
  {
   
  }
  
  if (pid < 0)
    {
      perror("fork error");
      exit(-1);
    }

}*/


int sql_doexecute(int a)
{
EXEC SQL BEGIN DECLARE SECTION;
char *buf,buf2[500],*buf3;
EXEC SQL END DECLARE SECTION;
GdkFont *fixed_font;

  EXEC SQL WHENEVER SQLERROR GOTO breakout;

  strcat(dyn_statement,"  ");
  buf=strtok(dyn_statement," ");
  buf=strtok(NULL," ");

  sprintf(buf2,"begin %s; end;",buf);
  EXEC SQL EXECUTE IMMEDIATE :buf2;

  fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
  gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), fixed_font, NULL, NULL,"PL/SQL procedure successfully completed.",40 );

  gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar1),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar1), ""), " ");
  gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar2),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar2), ""), " ");

 return;
	              
breakout:
 sql_sql_error(); 


}


void sql_clear_text (int a)
{ 
int i;

if(a){
  i = gtk_text_get_length (GTK_TEXT(sql_mainwin.sql_text5)); 
  gtk_text_freeze (GTK_TEXT (sql_mainwin.sql_text5));

  if (i > 0) {
    gtk_text_set_point (GTK_TEXT(sql_mainwin.sql_text5), i);
    gtk_text_backward_delete (GTK_TEXT(sql_mainwin.sql_text5), i);
  }
  gtk_text_thaw (GTK_TEXT (sql_mainwin.sql_text5));
  
}else{
  i = gtk_text_get_length (GTK_TEXT(sql_mainwin.sql_text4)); 
  gtk_text_freeze (GTK_TEXT (sql_mainwin.sql_text4));

  if (i > 0) {
    gtk_text_set_point (GTK_TEXT(sql_mainwin.sql_text4), i);
    gtk_text_backward_delete (GTK_TEXT(sql_mainwin.sql_text4), i);
  }
  gtk_text_thaw (GTK_TEXT (sql_mainwin.sql_text4));    
}  
}


void sql_discovery(void)
{
FILE *stream;
char buf[255],*buf2,*homedir,buf3[255];
int a=0,undo=0;

homedir=getenv("HOME");
sprintf(buf,"ls %s/.oralinux/*.reg",homedir);

stream=popen(buf,"r");
 while(fgets(buf3,255,stream)!=NULL){
  
  sql_disc[a].dname=(char *)malloc(255);
  sql_disc[a].path=(char *)malloc(255);
  sql_disc[a].icon=(char *)malloc(255);

  strtok(buf3,"\r\n");
  
  buf2=rc_getval("name=",buf3);
  if(buf2!=NULL)
    strcpy(sql_disc[a].dname,buf2);
  else
    undo=1;

  buf2=rc_getval("exepath=",buf3);  
  if(buf2!=NULL)
    strcpy(sql_disc[a].path,buf2);
  else
    undo=1;
    
  buf2=rc_getval("icon=",buf3);  
  if(buf2!=NULL)  
    strcpy(sql_disc[a].icon,buf2);
  else
    undo=1;
        
  if(undo) {
    undo=0;
    free(sql_disc[a].dname);
    free(sql_disc[a].path);
    free(sql_disc[a].icon);
    a--;

  }
  a++;
 }
 sql_ginfo.totaldisc = a;

}

void sql_rebuildclist(){

  gtk_widget_destroy(sql_mainwin.sql_clist2);

  sql_mainwin.sql_clist2 = gtk_clist_new (2);
  gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clist2", sql_mainwin.sql_clist2);
  gtk_widget_show (sql_mainwin.sql_clist2);
  gtk_container_add (GTK_CONTAINER (sql_mainwin.sql_scrolledwindow2), sql_mainwin.sql_clist2);
  gtk_clist_set_column_width (GTK_CLIST (sql_mainwin.sql_clist2), 0, 300);
  gtk_clist_set_column_width (GTK_CLIST (sql_mainwin.sql_clist2), 1, 10);
  gtk_clist_column_titles_show (GTK_CLIST (sql_mainwin.sql_clist2));
  gtk_widget_add_events(sql_mainwin.sql_clist2, GDK_ALL_EVENTS_MASK);
  gtk_signal_connect (GTK_OBJECT (sql_mainwin.sql_clist2), "draw",
                      GTK_SIGNAL_FUNC (sql_on_clist2_draw),
                      NULL);

}

int sql_getobjtype(char *owner,char *objname, char *dblink)
{
EXEC SQL BEGIN DECLARE SECTION;
char *buf,buf2[20],objowner[255],sqlstatement[5000];
EXEC SQL END DECLARE SECTION;

EXEC SQL WHENEVER SQLERROR CONTINUE;

buf=strupr(objname);
strcpy(objowner,strupr(owner));
buf2[0]=0;

  sprintf(sqlstatement,"SELECT object_type "
                       "FROM all_objects%s WHERE object_name = '%s' "
                       "AND owner = '%s'",dblink,buf,objowner);

  printf("\n%s",sqlstatement);
  EXEC SQL PREPARE firstpassA FROM :sqlstatement;	     

  exec sql declare firstpassB cursor for firstpassA;
  exec sql open firstpassB;
  
  EXEC SQL FETCH firstpassB INTO :buf2;

if(buf2[0]==0){
    
    sprintf(sqlstatement,"SELECT object_type "
                         "FROM all_objects%s WHERE object_name = '%s' "
                         "AND owner = 'PUBLIC'",dblink,buf);

    printf("\n%s",sqlstatement);
    EXEC SQL PREPARE firstpassC FROM :sqlstatement;	     

    exec sql declare firstpassD cursor for firstpassC;
    exec sql open firstpassD;
  
    EXEC SQL FETCH firstpassD INTO :buf2;
    
    if(buf2[0]==0){
      return 0;
    }else{
      
      if(strcmp(sse(buf2),"TABLE")==0)
        return 4;
      
      if(strcmp(sse(buf2),"SYNONYM")==0)
        return 5;
	
      if(strcmp(sse(buf2),"VIEW")==0)
        return 6;
      
    }
  
}else{

      if(strcmp(sse(buf2),"TABLE")==0)
        return 1;
	
      if(strcmp(sse(buf2),"SYNONYM")==0)
        return 2;
	
      if(strcmp(sse(buf2),"VIEW")==0)
        return 3;

}

return 0;

}


int sql_dodescribe()
{
EXEC SQL BEGIN DECLARE SECTION;
char *buf6,buf3[255],*buf4,*buf5,syntable[255],synowner[255];
int objtype=0;
varchar ltext[30000];
char displaytext[30500],tmplogin[500],*buf7,*tmpdblink=NULL,dblink[255];
EXEC SQL END DECLARE SECTION;

  strcpy(tmplogin,sql_ginfo.globallogin);
  strcat(dyn_statement," ");
  strtok(dyn_statement,"\r\n");
  buf6=strtok(dyn_statement," ");
  buf6=strtok(NULL," ");
  strcpy(buf3,buf6);
  buf7=buf6;

  if(strstr(buf6,"@")!=NULL){
     buf4=strtok(buf7,"@");
     tmpdblink=strtok(NULL," "); 
  }
  
  if(tmpdblink!=NULL)
    sprintf(dblink,"@%s",tmpdblink);
  else
    sprintf(dblink," ");

  buf4=NULL;
  if(strstr(buf6,".")!=NULL){
    buf4=strtok(buf6,".");
    buf5=strtok(NULL," ");
    objtype=sql_getobjtype(buf4,buf5,dblink);
  }else{
    buf5=buf3;
    buf4=strtok(tmplogin,"/");
    objtype=sql_getobjtype(buf4,buf5,dblink);
  }


  switch (objtype){
  
    case 1:
        sql_describetable(buf4,buf5,dblink);
      break;
    case 2:    
        EXEC SQL SELECT table_name,table_owner 
	  INTO :syntable,:synowner
	  FROM ALL_SYNONYMS
	  WHERE SYNONYM_NAME = :buf5 and OWNER = :buf4;
	  
	sprintf(displaytext,"%s is a Private Synonym for %s.%s",buf5,sse(synowner),sse(syntable)); 
	sql_showtext(displaytext);
      break;
    case 3:
      EXEC SQL SELECT text 
        INTO :ltext
	FROM all_views 
	WHERE view_name = :buf5 and owner = :buf4;
	
	ltext.arr[strlen(ltext.arr)+1]=0;
	sprintf(displaytext,"%s is a View of the following SQL:\n\n%s",buf5,ltext.arr);
	sql_showtext(displaytext);
      break;
      
    case 4:
        sql_describetable("PUBLIC",buf5,dblink);
      break;
    case 5:
        EXEC SQL SELECT table_name,table_owner 
	  INTO :syntable,:synowner
	  FROM ALL_SYNONYMS
	  WHERE SYNONYM_NAME = :buf5 and OWNER = 'PUBLIC';
	  
	sprintf(displaytext,"%s is a Public Synonym for %s.%s",buf5,sse(synowner),sse(syntable));  
	sql_showtext(displaytext);  
      break;
    case 6:
        EXEC SQL SELECT text INTO :ltext
	FROM all_views 
	WHERE view_name = :buf5 and owner = 'PUBLIC';
	
	sprintf(displaytext,"%s is a View of the following SQL:\n\n%s",buf5,ltext.arr);
	sql_showtext(displaytext);
	
      break;
   }   

   return 0;
}

void sql_showtext(char *distext)
{
char *buf[2];
sql_rebuildclist();
buf[0]=distext;
buf[1]=NULL;
gtk_clist_append( (GtkCList *) sql_mainwin.sql_clist2, buf);
gtk_text_set_word_wrap (GTK_TEXT(sql_mainwin.sql_text4),TRUE);
gtk_text_set_line_wrap (GTK_TEXT(sql_mainwin.sql_text4),TRUE);

gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), NULL, NULL, NULL,distext, strlen(distext));

}

void sql_describetable(char *owner,char *tablename,char *dblink)
{
EXEC SQL BEGIN DECLARE SECTION;
char buf[50000],buf2[500],buf3[500],colname[31],nullable[2],*buf4,*buf5,*buf6;
char dtatype[31],datalength[6],nullstring[12],dl[12],*gview[4];
EXEC SQL END DECLARE SECTION;
int i=0,reclen=0,tmprec=0,objtype=0;
GdkFont *fixed_font;

  sprintf(buf2,"select column_name,nullable,data_type,data_length from "
             "all_tab_columns%s where table_name = '%s' and owner = '%s'",dblink,tablename,owner);	  
	     
  EXEC SQL WHENEVER SQLERROR DO sql_sql_error();
  EXEC SQL PREPARE T FROM :buf2;	     

  exec sql declare msg11 cursor for T;
  exec sql open msg11;

  EXEC SQL WHENEVER NOTFOUND DO break;

  gtk_widget_destroy(sql_mainwin.sql_clist2);
  
  sql_mainwin.sql_clist2 = gtk_clist_new (4);
    gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clist2", sql_mainwin.sql_clist2);
    gtk_widget_show (sql_mainwin.sql_clist2);
    gtk_signal_connect (GTK_OBJECT (sql_mainwin.sql_clist2), "draw",
                      GTK_SIGNAL_FUNC (sql_on_clist2_draw),
                      NULL);


  i=0;
  sql_mainwin.sql_clistlabel[0] = gtk_label_new ("Field Name");
  sql_mainwin.sql_clistlabel[1] = gtk_label_new ("Null");
  sql_mainwin.sql_clistlabel[2] = gtk_label_new ("Data Type");
  sql_mainwin.sql_clistlabel[3] = gtk_label_new ("Length");

  while(i < 4){
        gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clistlabel[i]", sql_mainwin.sql_clistlabel[i]);
        gtk_widget_show (sql_mainwin.sql_clistlabel[i]);
        gtk_clist_set_column_widget (GTK_CLIST (sql_mainwin.sql_clist2), i, sql_mainwin.sql_clistlabel[i]);
  i++;
  }

  buf[0]=0;
  i=0;

  for(;;) {
    EXEC SQL FETCH msg11 
             INTO :colname,:nullable,:dtatype,:datalength;
    
     i++;
     if(strncmp(nullable,"Y",1)==0)
       strcpy(nullstring,"        ");
     else
       strcpy(nullstring,"NOT NULL");
   
     if(strncmp(dtatype,"V",1)==0 || strncmp(dtatype,"C",1)==0)
        sprintf(dl,"(%s)",sse(datalength));
     else 
        strcpy(dl," ");
        
     gview[0]=NULL;
     gview[1]=NULL;
     gview[2]=NULL;
     gview[3]=NULL;
	
     gview[0]=sse(ssb(colname));
     gview[1]=nullstring;
     gview[2]=sse(ssb(dtatype));
     gview[3]=dl;
     
     gtk_clist_append( (GtkCList *) sql_mainwin.sql_clist2, gview);
                        
     sprintf(buf3,"%-30.30s %-15.15s %s%s\n",colname,nullstring,dtatype,dl);
     printf("\n%s",buf3);
     fflush(stdout);
     
     tmprec=strlen(buf3);
     if(tmprec > reclen)
       reclen = tmprec;
     strcat(buf,buf3);
     
  }

    EXEC SQL WHENEVER NOTFOUND CONTINUE;
    EXEC SQL CLOSE msg11;
  
    fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
    strcat(buf,"\n\n");
    i += 2;
        
    sql_clear_text(0);
    gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), fixed_font, NULL, NULL,buf, strlen(buf));
    gtk_widget_set_usize (sql_mainwin.sql_text4, (reclen * 9), -1);
    i = gtk_text_get_length (GTK_TEXT(sql_mainwin.sql_text4)); 
    gtk_text_set_point (GTK_TEXT(sql_mainwin.sql_text4), i);  
      
    gtk_container_add (GTK_CONTAINER (sql_mainwin.sql_scrolledwindow2), sql_mainwin.sql_clist2);
    gtk_clist_column_titles_show (GTK_CLIST (sql_mainwin.sql_clist2));
    
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar1),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar1), ""), " ");
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar2),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar2), ""), " ");

 return;

}

int sql_viewtable()
{
char *buf6,buf3[255],*buf4,*buf5,tmplogin[500];
char *buf[4],*buf7,buf8[255],tbregpath[500],*tmpchar;
int objtype=0;
pid_t pid;

  strcpy(tmplogin,sql_ginfo.globallogin);
  strcat(dyn_statement," ");
  strtok(dyn_statement,"\r\n");
  buf6=strtok(dyn_statement," ");
    
  buf6=strtok(NULL," ");

  if(buf6==NULL)
    return 0;
      
  strcpy(buf3,buf6);

  if(strstr(buf6,".")!=NULL){
    buf4=strtok(buf6,".");
    buf5=strtok(NULL," ");
    objtype=sql_getobjtype(buf4,buf5," ");
  }else{
    buf5=buf3;
    buf4=strtok(tmplogin,"/");
    objtype=sql_getobjtype(buf4,buf5," ");
  }

  sprintf(buf8,"%s.%s",buf4,buf5);
  sprintf(tbregpath,"%s/.oralinux/tablerc.reg",getenv("HOME"));

  buf7=rc_getval("exepath=",tbregpath);
  
  if(buf7!=NULL){
    pid = fork();

    if (pid == 0)
    {

      buf[0]=buf7;
      buf[1]=sql_ginfo.globallogin;
      buf[2]=buf8;
      buf[3]=NULL;
      printf("\n0=%s\n1=%s\n2=%s\n",buf[0],buf[1],buf[2]);

      execv(buf7, buf);
        perror("exec error");
        exit(-1);
    }
    if (pid < 0)
      {
        perror("fork error");
        exit(-1);
      }
  }
  return(0);
}

/* Not used Yet */
int getsessionid()
{
char sqlstatement[255],randseed[7];
time_t t;
struct tm *tm;

t=time(NULL);
tm=localtime(&t);

sprintf(randseed,"%02d%02d%02d",tm->tm_hour,tm->tm_min,tm->tm_sec); 
sprintf(sqlstatement,"select '%s' from dual",randseed);

}

void sql_showsql()
{

if(sql_ginfo.totalhistory){

  if(sql_ginfo.historypointer == 20)
    sql_ginfo.historypointer = 19;
  
  if(sql_ginfo.historypointer == -1)
    sql_ginfo.historypointer = 0;

  if(sql_ginfo.historypointer >= sql_ginfo.totalhistory)
    sql_ginfo.historypointer = sql_ginfo.totalhistory -1;

  sql_clear_text(1);

  gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text5), NULL, NULL, NULL,sql_ginfo.sqlhistory[sql_ginfo.historypointer], 
                                                    strlen(sql_ginfo.sqlhistory[sql_ginfo.historypointer]));


}
}

void do_regfile(char *regfile)
{
struct stat sb,sb2;
char *homedir,buf[2551],regpath[2551],skelpath[2551];
FILE *stream,*stream2;

 homedir=getenv("HOME");
 sprintf(regpath,"%s/.oralinux/%s",homedir,regfile);
 sprintf(skelpath,"/usr/local/share/orasoft/skel/%s",regfile);

 if (stat(regpath, &sb) == -1 && stat(skelpath, &sb2) != -1){
   stream=fopen(skelpath,"r");
   stream2=fopen(regpath,"w");
   if(stream==NULL || stream2==NULL){
     printf("Could not copy skeleton file to HOME\n");
     return;
   }else{
     while(fgets(buf,2550,stream)!=NULL)
       fputs(buf,stream2);
   }
   fclose(stream);
   fclose(stream2);
 }
}   

