/*
 * Copyright ? Matthew Chappee <matthew@orasoft.org>
 *             Paul Flinders <paul@orasoft.org>
 *             Brent Gilmore <brent@orasoft.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include "sqlworksrc.h"
#include "sqlworksig.h"
#include <stdlib.h>
#include <stdio.h>
#include <strings.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sqlcpr.h>
#include <time.h>
#include <errno.h>

#define MAX_ITEMS         300
#define MAX_VNAME_LEN     30
#define MAX_INAME_LEN     30
#ifndef NULL
#define NULL  0
#endif

EXEC SQL INCLUDE sqlda;
EXEC SQL INCLUDE sqlca;
SQLDA *bind_dp;
SQLDA *select_dp;

EXEC SQL BEGIN DECLARE SECTION;
   char    dyn_statement[1024];
   EXEC SQL VAR dyn_statement IS STRING(1024);
   const char *sql_cstring;
EXEC SQL END DECLARE SECTION;

int sql_parse_flag = 0,sql_errorflag = 0,sql_h = 0;
static int lengths[] = { -1, 0, 45, 0, 0, 0, 0, 0, 255, 0, 0, 18, 25, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 512, 255 };

char *dml_commands[] = {"SELECT", "select", "INSERT", "insert","CONNECT","connect",
                        "UPDATE", "update", "DELETE", "delete","EXECUTE","execute",
			"EXEC","exec"};


int
main (int argc, char *argv[])
{
  FILE *stream;
  char buf[255],sqlfile[255];
  EXEC SQL WHENEVER SQLERROR DO sql_error();
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  do_regfile("sqlworkrc.reg");

  sql_initglobals();
  /*sql_ginfo.sessionid=getsessionid();*/

  sql_mainwin.sql_window1 = sql_create_window1();  
  sql_loginwin.sql_login = sql_create_login();
  
  if(argv[1]){
    strcpy(sql_ginfo.globallogin,argv[1]);
    gtk_widget_show (sql_mainwin.sql_window1);    
    sql_on_login_ok_clicked(NULL, NULL);
    
    if(argv[2]){
      if(argv[2][0]=='@'){
         strcpy(sqlfile,argv[2]+1);
         stream=fopen(sqlfile,"r");
	 if(stream!=NULL){
	   while(fgets(buf,1000,stream)!=NULL)
	     gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text5), NULL, NULL, NULL,buf, strlen(buf));
	 }
      }else{
        gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text5), NULL, NULL, NULL,argv[2], strlen(argv[2]));
        sql_on_button9_clicked(NULL,NULL);
      }
    }
  }else{
    gtk_widget_show (sql_loginwin.sql_login);
  }            
    
  gtk_main ();
  return 0;
}


gboolean
sql_on_window1_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{

  switch (event->keyval){
  
    case 65470:
      sql_on_button1_clicked(NULL,NULL);
      break;
      
    case 65471:
      sql_on_button2_clicked(NULL,NULL);
      break;
      
    case 65472:
      sql_on_button3_clicked(NULL,NULL);
      break;
      
    case 65473:
      sql_on_button4_clicked(NULL,NULL);
      break;
      
    case 65479:
      sql_on_button5_clicked(NULL,NULL);
      break;
  
    case 65475:
      sql_on_button9_clicked(NULL,NULL);
      break;
    
    case 65476:
      sql_on_button7_clicked(NULL,NULL);
      break;
      
    case 65477:
      sql_on_button8_clicked(NULL,NULL);
      break;

    case 65363:
      if(event->state==4){
        sql_ginfo.historypointer++;
        sql_showsql();
      }
      break;

    case 65361:
      if(event->state==4){
        sql_ginfo.historypointer--;
        sql_showsql();
      }
      break;
   
  } 
/*  printf("\nkeyval=%d\nstate=%d\nlength=%d\nstring=%s",event->keyval,event->state,
          event->length,event->string);
*/
  return TRUE;
}

void
sql_on_Add_Bookmark_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}

void
sql_on_Edit_Bookmark_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}


void
sql_on_Open_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
sql_create_fileselection();
gtk_widget_show (sql_filew);
}


void
sql_on_Save_Query_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
FILE *stream;
char *cp;

  if(strncmp(sql_ginfo.filename,"none",4)==0){
    sql_on_Save_Query_As_activate(menuitem,user_data);
  }else{

    cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text5)));
          if(cp==NULL){
            printf("\r\nOut of Memory\r\n");
            exit(0);
          }

    cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text5),(gint) 0,
      (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text5)));
    
    stream=fopen(sql_ginfo.filename,"w");
    if (stream) {
	fprintf(stream,"%s",cp);
	fclose(stream);
    } else {
	file_open_error(sql_ginfo.filename, errno, "write");
    }
  
    free(cp);

  }
}


void
sql_on_Save_Query_As_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  sql_create_filesave(1);
  gtk_widget_show (sql_filesave);
}


void
sql_on_Quit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

sql_destroy(NULL,user_data);

}


void
sql_on_Save_Connection_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
int a;

if(strncmp(sql_ginfo.globallogin,"none",4)!=0)
  a=rc_changeval("login=",sql_ginfo.globallogin);
}


void
sql_on_Forget_Connection_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
int a;

if(strncmp(sql_ginfo.globallogin,"none",4)!=0)
  a=rc_changeval("login=","");
}


void
sql_on_Change_Connection_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show (sql_loginwin.sql_login);
  gtk_widget_hide (sql_mainwin.sql_window1);
}

void
sql_on_Change_Delimiter_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  sql_create_window5(1);
  gtk_entry_set_text(GTK_ENTRY(sql_configwin.sql_entry10),sql_ginfo.globaldelim);
  gtk_widget_show (sql_configwin.sql_window5);
}										
					
void
sql_on_Change_Max_Buffer_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  sql_create_window5(0);
  gtk_entry_set_text(GTK_ENTRY(sql_configwin.sql_entry10),sql_ginfo.globalbuffer);
  gtk_widget_show (sql_configwin.sql_window5);
}										

void
sql_on_Discovery_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
pid_t pid;
char *buf[3];
struct stat sb;

  if (stat(sql_disc[(int)user_data].path, &sb) == -1){
      printf("\nFile %s is not an executable file",sql_disc[(int)user_data].path);
      return;
  }

  pid = fork();

  if (pid == 0)
  {

    buf[0]=sql_disc[(int)user_data].path;
    buf[1]=sql_ginfo.globallogin;
    buf[2] = NULL;

    execv(sql_disc[(int)user_data].path, buf);
      perror("exec error");
      exit(-1);
  }
  if (pid < 0)
    {
      perror("fork error");
      exit(-1);
    }

}


void
sql_on_About_SQLWork_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
char abouttext[500];

sprintf(abouttext,"\nAbout SQLWork:\n\nVersion 3.1\n"
                  "\n\nSQLWork is a SQL editor for Linux.  It will,\n of course, work on "
		  "all major Unix platforms.\n\nSQLWork is being actively developed by Matthew Chappee "
		  "\n(matthew@mattshouse.com).");
sql_create_aboutbox();
gtk_widget_show (sql_aboutwin.sql_aboutbox);
gtk_text_insert (GTK_TEXT (sql_aboutwin.sql_text3), NULL, NULL, NULL,abouttext, strlen(abouttext));
}


void
sql_on_About_OraLinux_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
char abouttext[500];

sprintf(abouttext,"\nAbout OraLinux:\n\nOralinux is an ongoing project to bring enterprise\nquality "
                  "database software to the Linux platform.  If you want to\nhelp with development, "
		  "documentation, or artwork please send me\nmail at matthew@mattshouse.com.");
sql_create_aboutbox();
gtk_widget_show (sql_aboutwin.sql_aboutbox);
gtk_text_insert (GTK_TEXT (sql_aboutwin.sql_text3), NULL, NULL, NULL,abouttext, strlen(abouttext));
}

static void sql_menuitem_response (gchar *string)
{
    sql_clear_text(1);
    gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text5), NULL, NULL, NULL,sql_ginfo.sqlhistory[atoi(string)], 
                                               strlen(sql_ginfo.sqlhistory[atoi(string)]));
}

gboolean sql_on_text5_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
GtkWidget *menu_items;
GtkWidget *menu;
int i=0;
char buf[51],tmpnum[255];


 if (event->button==3 && sql_ginfo.totalhistory > 0){

    menu = gtk_menu_new();
    while(i < sql_ginfo.totalhistory)
        {
            strncpy(buf, sql_ginfo.sqlhistory[i], 49);
	    buf[49]='\0';
            menu_items = gtk_menu_item_new_with_label(buf);
            gtk_menu_append(GTK_MENU (menu), menu_items);
	    sprintf(tmpnum,"%d",i);
	    gtk_signal_connect_object(GTK_OBJECT(menu_items), "activate",
		GTK_SIGNAL_FUNC(sql_menuitem_response), (gpointer) g_strdup(tmpnum));

            gtk_widget_show(menu_items);
	    i++;
        }
	
     gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
                        event->button, event->time);

 }
 
}

void
sql_on_button1_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
sql_clear_text(0);
sql_clear_text(1);
}


void
sql_on_button2_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
sql_create_fileselection();
gtk_widget_show (sql_filew);
}


void
sql_on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
FILE *stream;
char *cp,*buf1;

  if(strncmp(sql_ginfo.filename,"none",4)==0){
    sql_on_Save_Query_As_activate(NULL,NULL);
  }else{

    cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text5)));
          if(cp==NULL){
            printf("\r\nOut of Memory\r\n");
            exit(0);
          }

    cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text5),(gint) 0,
      (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text5)));
    
    stream=fopen(sql_ginfo.filename,"w");
    if (stream) {
	fprintf(stream,"%s",cp);
	fclose(stream);
    } else {
	file_open_error(sql_ginfo.filename, errno, "write");
    }
  
    free(cp);
  }
}


void
sql_on_button4_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
char buf[255],*buf1;

sql_ginfo.globalprint = 0;
sql_printwin.sql_window3 = sql_create_window3();

sprintf(buf,"lpr -J \"Sqlwork Query\" -T \"Sqlwork Query\"",buf1,buf1);
gtk_widget_show (sql_printwin.sql_window3);
gtk_entry_set_text(GTK_ENTRY(sql_printwin.sql_entry1),buf);
}


void
sql_on_button5_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
sql_destroy(NULL,user_data);
}


void
sql_on_button7_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
char buf[255],*buf1;

sql_printwin.sql_window3 = sql_create_window3();
sql_ginfo.globalprint=1;

sprintf(buf,"lpr -J \"Sqlwork Output\" -T \"Sqlwork Output\"",buf1,buf1);
gtk_widget_show (sql_printwin.sql_window3);
gtk_entry_set_text(GTK_ENTRY(sql_printwin.sql_entry1),buf);
}


void
sql_on_button8_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
char *buf[2];

    buf[1]=(char *)malloc(255);
    buf[0]=(char *)malloc(255);

    buf[0]=NULL;
    
    sql_exportwin.sql_window4=sql_create_window4();
    
    sprintf(buf[1],"HTML (Star Office)");
    gtk_clist_append( (GtkCList *) sql_exportwin.sql_clist3, buf);
    free(buf[1]);
    
    buf[1]=(char *)malloc(255);
    sprintf(buf[1],"CSV (Applix)");
    gtk_clist_append( (GtkCList *) sql_exportwin.sql_clist3, buf);
    free(buf[1]);
    
    buf[1]=(char *)malloc(255);
    sprintf(buf[1],"Tab Delimited (Gnumeric)");
    gtk_clist_append( (GtkCList *) sql_exportwin.sql_clist3, buf);
    free(buf[1]);
    
    buf[1]=(char *)malloc(255);
    sprintf(buf[1],"Use Text View Delimiter (\"%s\")",sql_ginfo.globaldelim);
    gtk_clist_append( (GtkCList *) sql_exportwin.sql_clist3, buf);
    free(buf[1]);
    free(buf[0]);
  
  gtk_widget_show (sql_exportwin.sql_window4);
}


void
sql_on_button9_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
   int i,t;

    if(sql_ginfo.doingquery)
      return;
      
    sql_ginfo.doingquery = 1;
    sql_clear_text(0);

    EXEC SQL WHENEVER SQLERROR GOTO breakout; 

    if (sql_alloc_descriptors(MAX_ITEMS, MAX_VNAME_LEN, MAX_INAME_LEN) != 0)
        exit(1);  
        
        /* Get the statement. */
	dyn_statement[0]=0;
        sql_get_dyn_statement();      
	     
	     
	     
	if(strncmp(dyn_statement,"table",5)==0 || strncmp(dyn_statement,"TABLE",5)==0){
	   sql_ginfo.doingquery = 0;
	   if(sql_viewtable())
	      goto breakout;
	    else
	       return;
	}
	     
	     
	if(strncmp(dyn_statement,"desc",4)==0 || strncmp(dyn_statement,"DESC",4)==0){
	   sql_ginfo.doingquery = 0;
	   if(sql_dodescribe())
	      goto breakout;
	    else
	       return;
	}
	
	if(strncmp(dyn_statement,"connect",7)==0 || strncmp(dyn_statement,"CONNECT",7)==0){
	   sql_ginfo.doingquery = 0;
	   if(sql_doconnect())
	      goto breakout;
	    else
	       return;
	}
     
        if(strncmp(dyn_statement,"exec",4)==0 || strncmp(dyn_statement,"EXEC",4)==0){
	   sql_ginfo.doingquery = 0;
	   if(sql_doexecute(0))
	      goto breakout;
	    else
	       return;
	}
         	    
/*	if(strncmp(dyn_statement,"fork",4)==0 || strncmp(dyn_statement,"FORK",4)==0){
	   sql_ginfo.doingquery = 0;
	   if(doforkexec())
	      goto breakout;
	    else
	       return;
	}*/
	          
		      
	printf("\nhere1");	       
			 
        /*parse_flag = 1;      Set a flag for sql_error(). */
        EXEC SQL PREPARE S FROM :dyn_statement;
        /*parse_flag = 0;      Unset the flag. */

        printf("\nhere2");

        EXEC SQL DECLARE C CURSOR FOR S;

        printf("\nhere3");

        sql_set_bind_variables();

        printf("\nhere4");

        EXEC SQL OPEN C USING DESCRIPTOR bind_dp;

        printf("\nhere5");

        sql_process_select_list();
	sql_ginfo.doingquery = 0;
							
        sqlclu(bind_dp);
        sqlclu(select_dp); 

        EXEC SQL WHENEVER SQLERROR CONTINUE; 
        EXEC SQL CLOSE C;

        /*EXEC SQL COMMIT WORK;*/
        EXEC SQL WHENEVER SQLERROR DO sql_error();
      
  return;
	              
breakout:
 sql_sql_error();       

}

void
sql_on_button10_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  sql_ginfo.globalbreak = 1;
}


void
sql_on_button12_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_destroy(sql_printwin.sql_window3);
}										


void
sql_on_button13_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
    char buf2[255],*cp;
    FILE *stream;
    char tempfile[]="/tmp/sqltempXXXXXX";

    if(sql_ginfo.globalprint){
	cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text4)));
        if(cp==NULL){
	    printf("\r\nOut of Memory\r\n");
	    exit(0);
        }

	cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text4),(gint) 0,
				    (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text4)));

    }else{
	cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text5)));
        if(cp==NULL){
	    printf("\r\nOut of Memory\r\n");
	    exit(0);
        }

	cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text5),(gint) 0,
				    (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text5)));


    }

    mktemp(tempfile);

    stream=fopen(tempfile,"w");
    if (stream) {
	fprintf(stream,cp);
	fclose(stream);

	sprintf(buf2,"cat %s | %s",tempfile,gtk_entry_get_text(GTK_ENTRY(sql_printwin.sql_entry1)));
	system(buf2);
    } else {
	file_open_error(tempfile, errno, "write");
    }

    free(cp);
    gtk_widget_destroy(sql_printwin.sql_window3);

}										

void
sql_on_button14_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_destroy(sql_printwin.sql_window3);
}

void
sql_on_button15_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_destroy(sql_exportwin.sql_window4);
}


void
sql_on_button16_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
char outfile[255];

 gtk_widget_destroy(sql_exportwin.sql_window4);
 sql_create_filesave(0);
 if(sql_ginfo.crow!=-1){
   if(sql_ginfo.crow==0)
     strcpy(outfile,"output.html");
   if(sql_ginfo.crow==1)
     strcpy(outfile,"output.csv");
   if(sql_ginfo.crow==2)
     strcpy(outfile,"output.txt"); 
   if(sql_ginfo.crow==3)
     strcpy(outfile,"output.del");  
   
   gtk_file_selection_set_filename (GTK_FILE_SELECTION(sql_filesave),outfile);
   gtk_widget_show (sql_filesave);
 }
}

void
sql_on_button17_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
gtk_widget_destroy(sql_configwin.sql_window5);
}


void
sql_on_button18a_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
char tmpdelim[256];

strcpy(sql_ginfo.globaldelim,gtk_entry_get_text(GTK_ENTRY(sql_configwin.sql_entry10)));
sprintf(tmpdelim,"%s;",sql_ginfo.globaldelim);

rc_changeval("delim=",tmpdelim);
gtk_widget_destroy(sql_configwin.sql_window5);
}

void
sql_on_button18b_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
strcpy(sql_ginfo.globalbuffer,gtk_entry_get_text(GTK_ENTRY(sql_configwin.sql_entry10)));
rc_changeval("maxbuffer=",sql_ginfo.globalbuffer);
gtk_widget_destroy(sql_configwin.sql_window5);
}

void
sql_on_button19_clicked                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide (sql_mainwin.sql_text4);
  gtk_text_freeze (GTK_TEXT (sql_mainwin.sql_text4));
}

void
sql_on_text1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
gtk_toolbar_set_style(GTK_TOOLBAR(sql_mainwin.sql_toolbar1), GTK_TOOLBAR_TEXT);
rc_changeval("tbstate=","1");
}


void
sql_on_icons1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
gtk_toolbar_set_style(GTK_TOOLBAR(sql_mainwin.sql_toolbar1), GTK_TOOLBAR_ICONS);
rc_changeval("tbstate=","2");
}


void
sql_on_text_and_icons1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
gtk_toolbar_set_style(GTK_TOOLBAR(sql_mainwin.sql_toolbar1), GTK_TOOLBAR_BOTH);
rc_changeval("tbstate=","3");
}

void sql_on_login_cancel_clicked (GtkButton *button,gpointer user_data)
{
  gtk_main_quit();
}

sql_on_login_key_press_event             (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{

if(event->keyval==65293){
  sql_on_login_ok_clicked(GTK_BUTTON (sql_loginwin.sql_login_cancel),NULL);
}

}

void sql_on_login_ok_clicked (GtkButton *button,gpointer user_data)
{
EXEC SQL BEGIN DECLARE SECTION;
char *cstring,*user,*pass,*tns;
EXEC SQL END DECLARE SECTION;
          
if(!button){
  cstring=(char *)malloc(255);
  strcpy(cstring,sql_ginfo.globallogin);
}else{	  
  cstring=(char *)malloc(255);
  strcpy(cstring,gtk_entry_get_text(GTK_ENTRY(sql_loginwin.sql_username_entry)));
  if(strstr(cstring,"/")==NULL){
    user=cstring;
    pass=gtk_entry_get_text(GTK_ENTRY(sql_loginwin.sql_password_entry));
    tns=gtk_entry_get_text(GTK_ENTRY(sql_loginwin.sql_tns_entry));
    sprintf(cstring,"%s/%s@%s",user,pass,tns);
  }
  strcpy(sql_ginfo.globallogin,cstring);
}
           
EXEC SQL WHENEVER SQLERROR DO sql_sql_error();				                                 
EXEC SQL CONNECT :cstring;
    
free(cstring);    
    
sql_do_post_login();    
    
gtk_widget_show (sql_mainwin.sql_window1);
gtk_widget_hide (sql_loginwin.sql_login);

}	

void
sql_on_text4_draw                         (GtkWidget       *widget,
                                        GdkRectangle    *area,
                                        gpointer         user_data)
{
gtk_widget_show (sql_mainwin.sql_vscrollbar1);
}


void
sql_on_clist2_draw                         (GtkWidget       *widget,
                                        GdkRectangle    *area,
                                        gpointer         user_data)
{
gtk_widget_hide (sql_mainwin.sql_vscrollbar1);
}


void
sql_on_clist3_select_row                   (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  sql_ginfo.crow = row;
}
							

void sql_destroy( GtkWidget *widget,gpointer   data )
{
  gtk_main_quit();
}  

int sql_do_post_login()
{

}


void sql_file_cancel_sel (GtkWidget *w, GtkFileSelection *fs)
{

    gtk_widget_destroy(w);
}

void sql_destroy_about                      (GtkButton       *button,
                                        gpointer         user_data)
{

    gtk_widget_destroy(sql_aboutwin.sql_aboutbox);
}

void sql_file_ok_sel (GtkWidget *w, GtkFileSelection *fs)
{
gchar *nfile;
gchar *flash;
struct stat sb;
GdkFont *fixed_font;
int fd;
gchar *buf, *title;
size_t size, bytesread, num;

/* Borrowed from gEdit 'commands.c' and 'gE_files.c' */

        sql_ginfo.filename = g_malloc (strlen (gtk_file_selection_get_filename (GTK_FILE_SELECTION(sql_filew))) + 1);
        sql_ginfo.filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(sql_filew)));
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filew),"");

        if (sql_ginfo.filename != NULL)
          {
                if (stat(sql_ginfo.filename, &sb) == -1){
		        strncpy(sql_ginfo.filename,"none",4);
                        return;
                }
		 
                if (S_ISDIR(sb.st_mode))
                  {
                        nfile = g_malloc0(strlen (sql_ginfo.filename) + 3);
                        sprintf(nfile, "%s/.", sql_ginfo.filename);
                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filew), nfile);
                        g_free(nfile);
			strncpy(sql_ginfo.filename,"none",4);
                        return;
                  }       
		  
               if ((fd = open(sql_ginfo.filename, O_RDONLY)) == -1) {
		
			printf("SQLWork could not open %s", sql_ginfo.filename);
			strncpy(sql_ginfo.filename,"none",4);
			return;
		}

		gtk_text_freeze(GTK_TEXT (sql_mainwin.sql_text5));
		fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
		sql_clear_text(1);

		size = 2 * (sb.st_size + 1);
		while ((buf = (char *)g_malloc(size)) == NULL)
			size /= 2;

		g_free(buf);
		size /= 2;
		if ((buf = (char *)g_malloc(size + 1)) == NULL) {
			perror("Out of memory");
			close(fd);
			strncpy(sql_ginfo.filename,"none",4);
			return;
		}

		bytesread = 0;
		while (bytesread < sb.st_size) {
			num = read(fd, buf, size);
			if (num > 0) {
				gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text5), fixed_font, NULL,NULL, buf, num);
				bytesread += num;
			} else if (num == -1) {
				/* need to print warning/error message */
				perror("read error");
				break;
			} else /* if (num == 0) */ {

			}
		}
		g_free(buf);
		close(fd);
		gtk_text_thaw(GTK_TEXT(sql_mainwin.sql_text5));
	
    }
    gtk_widget_destroy(sql_filew);
    
}

void sql_file_ok_save (GtkWidget *w, GtkFileSelection *fs)
{
gchar *nfile;
gchar *flash;
struct stat sb;
GdkFont *fixed_font;
int fd;
gchar *buf, *title;
size_t size, bytesread, num;

/* Borrowed from gEdit 'commands.c' and 'gE_files.c' */

        sql_ginfo.filename = g_malloc (strlen (gtk_file_selection_get_filename (GTK_FILE_SELECTION(sql_filesave))) + 1);
        sql_ginfo.filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(sql_filesave)));
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filesave),"");

        if (sql_ginfo.filename != NULL)
          {

                if (S_ISDIR(sb.st_mode))
                  {
                        nfile = g_malloc0(strlen (sql_ginfo.filename) + 3);
                        sprintf(nfile, "%s/.", sql_ginfo.filename);
                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filew), nfile);
                        g_free(nfile);
			strncpy(sql_ginfo.filename,"none",4);
                        return;
                  }      	  
	
	    sql_on_Save_Query_activate(NULL,NULL);
	
        }
      gtk_widget_destroy(sql_filesave);
    
}

void sql_file_ok_export (GtkWidget *w, GtkFileSelection *fs)
{
gchar *nfile;
gchar *flash;
struct stat sb;
GdkFont *fixed_font;
int fd;
gchar *buf, *title;
size_t size, bytesread, num;

/* Borrowed from gEdit 'commands.c' and 'gE_files.c' */

        sql_ginfo.filename = g_malloc (strlen (gtk_file_selection_get_filename (GTK_FILE_SELECTION(sql_filesave))) + 1);
        sql_ginfo.filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(sql_filesave)));
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filesave),"");

        if (sql_ginfo.filename != NULL)
          {

                if (S_ISDIR(sb.st_mode))
                  {
                        nfile = g_malloc0(strlen (sql_ginfo.filename) + 3);
                        sprintf(nfile, "%s/.", sql_ginfo.filename);
                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filew), nfile);
                        g_free(nfile);
			strncpy(sql_ginfo.filename,"none",4);
                        return;
                  }      	  
	
	    if(sql_ginfo.crow==0)
	      sql_save_as_html(sql_ginfo.filename);
	    if(sql_ginfo.crow==1)
	      sql_save_as_csv(sql_ginfo.filename);
	    if(sql_ginfo.crow==2)
	      sql_save_as_tab(sql_ginfo.filename); 
	    if(sql_ginfo.crow==3)
	      sql_save_as_delim(sql_ginfo.filename); 
	
        }
      gtk_widget_destroy(sql_filesave);
    
}



void sql_output_file_ok_save (GtkWidget *w, GtkFileSelection *fs)
{
gchar *nfile;
gchar *flash;
struct stat sb;
GdkFont *fixed_font;
int fd;
gchar *buf, *title;
size_t size, bytesread, num;
FILE *stream;
char *cp,*buf1;


/* Borrowed from gEdit 'commands.c' and 'gE_files.c' */

        sql_ginfo.filename = g_malloc (strlen (gtk_file_selection_get_filename (GTK_FILE_SELECTION(sql_filesave))) + 1);
        sql_ginfo.filename = g_strdup (gtk_file_selection_get_filename(GTK_FILE_SELECTION(sql_filesave)));
        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filesave),"");

        if (sql_ginfo.filename != NULL)
          {

                if (S_ISDIR(sb.st_mode))
                  {
                        nfile = g_malloc0(strlen (sql_ginfo.filename) + 3);
                        sprintf(nfile, "%s/.", sql_ginfo.filename);
                        gtk_file_selection_set_filename(GTK_FILE_SELECTION(sql_filew), nfile);
                        g_free(nfile);
			strncpy(sql_ginfo.filename,"none",4);
                        return;
                  }      	  

	        cp=(char *)malloc(gtk_text_get_length(GTK_TEXT (sql_mainwin.sql_text4)));
	             if(cp==NULL){
	               printf("\r\nOut of Memory\r\n");
	              exit(0);
	             }
	
	        cp = gtk_editable_get_chars(GTK_EDITABLE (sql_mainwin.sql_text4),(gint) 0,
	        (gint) gtk_text_get_length (GTK_TEXT (sql_mainwin.sql_text4)));
	    
	       stream=fopen(sql_ginfo.filename,"w");
	       if (stream) {
		   fprintf(stream,"%s",cp);
		   fclose(stream);
	       } else {
		   file_open_error(sql_ginfo.filename, errno, "write");
	       }
	  
	       free(cp);

	
        }
      gtk_widget_destroy(sql_filesave);
    
}

void sql_initglobals(){
char *buf4,*homedir;

  sql_ginfo.totaldisc=0;
  sql_ginfo.crow = -1;
  sql_ginfo.globalbreak = 0;
  sql_ginfo.fastmode = 0;
  sql_ginfo.totalhistory = 0;
  sql_ginfo.historypointer = 0;
  sql_ginfo.doingquery = 0;
  sql_ginfo.freemem = 0;
  
  homedir=getenv("HOME");
  sprintf(sql_ginfo.regfile,"%s/.oralinux/sqlworkrc.reg",homedir);

  sql_ginfo.filename=(char *)malloc(255);
  strcpy(sql_ginfo.filename,"none");

  sql_ginfo.globallogin=(char *)malloc(255);
  strcpy(sql_ginfo.globallogin,"none");

  buf4=rc_getval("iconpath=",NULL);
  if(buf4!=NULL){
      strncpy(sql_ginfo.iconpath,buf4,strlen(buf4));
  }else{
      sprintf(sql_ginfo.iconpath,"%s/.oralinux/icons",homedir);
  }


  buf4=rc_getval("delim=",NULL);
  if(buf4!=NULL){
      strncpy(sql_ginfo.globaldelim,buf4,strlen(buf4) -1);
  }else{
      strcpy(sql_ginfo.globaldelim,"      ");
  }

  buf4=rc_getval("maxbuffer=",NULL);
  if(buf4!=NULL){
      strcpy(sql_ginfo.globalbuffer,buf4);
  }else{
      strcpy(sql_ginfo.globalbuffer,"100000");
  }

  buf4=rc_getval("login=",NULL);
  if(buf4!=NULL){
      strcpy(sql_ginfo.globallogin,buf4);
  }       

}

void sql_process_select_list()
{
    int i, null_ok, precision, scale,maxbuffer=0,a=0,l=0,cntr=0;
    int bb,scolor=0,b=0,j=0,k=350,reclen=0,tmplen=0,abswitch=1,fastflag=0;
    float progup=0.0;
    char *tmpbuf1,tmpbuf2[100000],numcol[100],delim[255];
    char *headr,*colnames,tcolnames[255],disptext[255];
    char *buf[1000], *buf2, *buf3[2];
    GdkColormap *cmap;
    GdkColor color;	
    GdkFont *fixed_font;

    maxbuffer=atoi(sql_ginfo.globalbuffer);
    if(maxbuffer==0)     			      
      maxbuffer=100000;			  
			          
    colnames=(char *)malloc(10000);
    if(colnames==NULL){
      printf("\r\nOut of Memory\r\n");
      exit(0);
     }

    colnames[0]=0;

    gtk_widget_destroy(sql_mainwin.sql_vscrollbar1);
    gtk_widget_destroy(sql_mainwin.sql_text4);
    gtk_widget_destroy(sql_mainwin.sql_clist2);
    	    
    sql_mainwin.sql_text4 = gtk_text_new (NULL, NULL);
    gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_text4", sql_mainwin.sql_text4);
    gtk_text_set_word_wrap (GTK_TEXT(sql_mainwin.sql_text4),FALSE);
    gtk_text_set_line_wrap (GTK_TEXT(sql_mainwin.sql_text4),FALSE);
    gtk_widget_show (sql_mainwin.sql_text4);
    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (sql_mainwin.sql_scrolledwindow1), sql_mainwin.sql_text4);
    gtk_signal_connect (GTK_OBJECT (sql_mainwin.sql_text4), "draw",
                      GTK_SIGNAL_FUNC (sql_on_text4_draw),
                      NULL);


    sql_mainwin.sql_vscrollbar1 = gtk_vscrollbar_new (GTK_TEXT (sql_mainwin.sql_text4)->vadj);
    gtk_box_pack_start (GTK_BOX (sql_mainwin.sql_hbox2), sql_mainwin.sql_vscrollbar1, FALSE,FALSE, 0);
    gtk_widget_show (sql_mainwin.sql_vscrollbar1);
    	    
    if ((strncmp(dyn_statement, "SELECT", 6) != 0) &&
        (strncmp(dyn_statement, "select", 6) != 0))
    {
        sprintf(numcol,"\n\n%d row%c processed.\n", sqlca.sqlerrd[2],
                       sqlca.sqlerrd[2] == 1 ? ' ' : 's');
        gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), NULL, NULL, NULL,numcol, strlen(numcol));
        select_dp->F = 0;
	
	sql_mainwin.sql_clist2 = gtk_clist_new (2);
        gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clist2", sql_mainwin.sql_clist2);
        gtk_widget_show (sql_mainwin.sql_clist2);
	gtk_container_add (GTK_CONTAINER (sql_mainwin.sql_scrolledwindow2), sql_mainwin.sql_clist2);
        gtk_signal_connect (GTK_OBJECT (sql_mainwin.sql_clist2), "draw",
                       GTK_SIGNAL_FUNC (sql_on_clist2_draw),
                      NULL);
	gtk_clist_set_column_width (GTK_CLIST (sql_mainwin.sql_clist2), 0, 200);
	gtk_clist_set_column_width (GTK_CLIST (sql_mainwin.sql_clist2), 1, 5);
	buf3[1]=NULL;
	buf3[0]=(char *)malloc(strlen(numcol)+1);
	strcpy(buf3[0],numcol);
	gtk_clist_append( (GtkCList *) sql_mainwin.sql_clist2, buf3);
	
	return;
    }

    /* If the SQL statement is a SELECT, describe the
        select-list items.  The DESCRIBE function returns
        their names, datatypes, lengths (including precision
        and scale), and NULL/NOT NULL statuses. */

    select_dp->N = MAX_ITEMS;
    
    EXEC SQL DESCRIBE SELECT LIST FOR S INTO select_dp;
    
    gtk_widget_show (sql_mainwin.sql_button10);
    gtk_widget_show (sql_mainwin.sql_button19);
    gtk_widget_show (sql_mainwin.sql_progressbar1);    
    gtk_widget_show (sql_mainwin.sql_vseparator2);
    gtk_widget_show (sql_mainwin.sql_vseparator3);
    
    sql_mainwin.sql_clist2 = gtk_clist_new (select_dp->F);
    gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clist2", sql_mainwin.sql_clist2);
    gtk_widget_show (sql_mainwin.sql_clist2);
    gtk_signal_connect (GTK_OBJECT (sql_mainwin.sql_clist2), "draw",
                      GTK_SIGNAL_FUNC (sql_on_clist2_draw),
                      NULL);


    /* If F is negative, there were more select-list
       items than originally allocated by sqlald(). */
    if (select_dp->F < 0)
    {
        printf ("\nToo many select-list items (%d), maximum is %d\n",
                -(select_dp->F), MAX_ITEMS);
        return;
    }

    /* Set the maximum number of array elements in the
       descriptor to the number found. */
    select_dp->N = select_dp->F;

    for (i = 0; i < select_dp->F; i++)
    {
        /* Turn off high-order bit of datatype */
        sqlnul ((unsigned short *)&(select_dp->T[i]), (unsigned short *)&(select_dp->T[i]), &null_ok);

          for (bb = 0; bb < select_dp->F; bb++)
           {
              sqlnul (&(select_dp->T[bb]), &(select_dp->T[bb]), &null_ok);

              if ( select_dp->T[bb] < sizeof(lengths)/sizeof(lengths[0]) )
              {
                      if ( lengths[select_dp->T[bb]] )
                                select_dp->L[bb]  = lengths[select_dp->T[bb]];
                      else select_dp->L[bb] += 5;
              }
              else select_dp->L[bb] += 5;

              select_dp->T[bb] = 5;
              select_dp->V[bb] = (char *)malloc( select_dp->L[bb] );
            }
				    
				    			    
        sprintf (tcolnames,"%-.*s ", select_dp->L[i], select_dp->S[i]);	
		
        sql_mainwin.sql_clistlabel[i] = gtk_label_new (strtok(tcolnames," "));
        gtk_object_set_data (GTK_OBJECT (sql_mainwin.sql_window1), "sql_mainwin.sql_clistlabel[i]", sql_mainwin.sql_clistlabel[i]);
        gtk_widget_show (sql_mainwin.sql_clistlabel[i]);
        gtk_clist_set_column_widget (GTK_CLIST (sql_mainwin.sql_clist2), i, sql_mainwin.sql_clistlabel[i]);
	
	strcat(colnames,tcolnames);
	strcat(colnames,sql_ginfo.globaldelim);
	      																				       					       	                                       
    }
    
    strcat(colnames,"\n");
    gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), NULL, &color, NULL,colnames, strlen(colnames));
    
    /* FETCH each row selected and print the column values. */
    EXEC SQL WHENEVER NOT FOUND GOTO end_select_loop;
    
    for (;;)
    {

        if(sql_ginfo.globalbreak)
	  break;
	  	
        EXEC SQL WHENEVER SQLERROR GOTO breakout;		
        EXEC SQL FETCH C USING DESCRIPTOR select_dp;

	j++;
	
         for (i = 0; i < select_dp->F; i++)
          {

            tmpbuf1=(char *)malloc(select_dp->L[i] + 100);
            sprintf (tmpbuf1,"%-*.*s ", (int)select_dp->L[i],(int)select_dp->L[i], select_dp->V[i]);
	            
	    b += strlen(sse(ssb(tmpbuf1)));

            if(b <= maxbuffer){	   
              sse(ssb(tmpbuf1));
	      gtk_clist_set_column_width (GTK_CLIST (sql_mainwin.sql_clist2), i, 80);
	      
	      buf[i]=(char *)malloc(strlen(tmpbuf1)+1);
	      sprintf(buf[i],"%s",tmpbuf1); 
		  
	      strcat(tmpbuf1,sql_ginfo.globaldelim);
	      strcat(tmpbuf2,tmpbuf1);
            }
	  
           free(tmpbuf1);
         }
	
         cmap = gdk_colormap_get_system();
             color.red = 0;
             color.green = 0;
	
	while (gtk_events_pending())
             gtk_main_iteration();
	     
        sprintf(disptext,"Current Row: %d",j);  
        gtk_progress_bar_update( GTK_PROGRESS_BAR(sql_mainwin.sql_progressbar1),progup);
        gtk_progress_set_format_string(GTK_PROGRESS(sql_mainwin.sql_progressbar1),disptext);
        gtk_progress_set_show_text(GTK_PROGRESS(sql_mainwin.sql_progressbar1),TRUE);

        if(cntr == 100)
	  abswitch=0;
	    
	if(cntr == 0)
	  abswitch=1; 
	  
        if(abswitch){
	   progup += .01;
	   cntr += 1;
        }else{
	   progup -= .01;
	   cntr-=1;
        }  


        if(b + strlen(colnames) <= maxbuffer){
	 if(scolor){
           color.blue = 0;
	   scolor=0;
        }else{
           color.blue = 0xffff;
	   scolor=1;
        }

        if (!gdk_color_alloc(cmap, &color))
             g_error("couldn't allocate color");
	     
	tmplen = strlen(tmpbuf2);
	if(tmplen > reclen)
	   reclen=tmplen + 1;
		   
	strcat(tmpbuf2,"\n");
	  	    	  
        fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");
	gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), fixed_font, &color, NULL,tmpbuf2, strlen(tmpbuf2));
	gtk_clist_append( (GtkCList *) sql_mainwin.sql_clist2, buf);
		
        }
	tmpbuf2[0]='\0';

    }
end_select_loop:
    
    sql_ginfo.fastmode = 0;
    sql_ginfo.globalbreak=0;			
    sprintf(numcol,"\n\n%d row%c processed.\n", j,
                       sqlca.sqlerrd[2] == 1 ? '\0' : 's');
        
    gtk_text_insert (GTK_TEXT (sql_mainwin.sql_text4), NULL, &color, NULL,numcol, strlen(numcol));
    gtk_text_thaw (GTK_TEXT (sql_mainwin.sql_text4));
    
    sprintf(numcol,"%d row%c processed", j,sqlca.sqlerrd[2] == 1 ? '\0' : 's');
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar1),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar1), ""), numcol);
    
    sprintf (numcol,"%d columns",(select_dp->F));
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar2),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar2), ""), numcol);

    gtk_widget_set_usize (sql_mainwin.sql_text4, (reclen * 9) + strlen(sql_ginfo.globaldelim), -1);
    i = gtk_text_get_length (GTK_TEXT(sql_mainwin.sql_text4)); 
    gtk_text_set_point (GTK_TEXT(sql_mainwin.sql_text4), i);
    gtk_editable_set_editable(GTK_EDITABLE (sql_mainwin.sql_text4),TRUE);
    gtk_editable_set_position(GTK_EDITABLE (sql_mainwin.sql_text4),i);
    gtk_widget_show (sql_mainwin.sql_text4);

    gtk_container_add (GTK_CONTAINER (sql_mainwin.sql_scrolledwindow2), sql_mainwin.sql_clist2);
    gtk_clist_column_titles_show (GTK_CLIST (sql_mainwin.sql_clist2));

    sql_cinfo.numcols = select_dp->F;
    sql_cinfo.numrows = j;
    gtk_widget_hide (sql_mainwin.sql_button10);
    gtk_widget_hide (sql_mainwin.sql_button19);
    gtk_widget_hide (sql_mainwin.sql_progressbar1);
    gtk_widget_hide (sql_mainwin.sql_vseparator2);
    gtk_widget_hide (sql_mainwin.sql_vseparator3);

    free(colnames);
    sql_ginfo.doingquery = 0;
    EXEC SQL WHENEVER SQLERROR DO sql_sql_error();
    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    
    return;

breakout:
    sql_sql_error();
    gtk_widget_hide (sql_mainwin.sql_button10);
    gtk_widget_hide (sql_mainwin.sql_button19);
    gtk_widget_hide (sql_mainwin.sql_progressbar1);
    gtk_widget_hide (sql_mainwin.sql_vseparator2);
    gtk_widget_hide (sql_mainwin.sql_vseparator3);

    EXEC SQL WHENEVER SQLERROR DO sql_sql_error();
    EXEC SQL WHENEVER NOT FOUND CONTINUE;
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar1),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar1), ""), " ");
    gtk_statusbar_push( GTK_STATUSBAR(sql_mainwin.sql_statusbar2),gtk_statusbar_get_context_id(GTK_STATUSBAR(sql_mainwin.sql_statusbar2), ""), " ");

    free(colnames);
    sql_ginfo.doingquery = 0;       
}

